/* xns_main.c - main loop and graphics code for xnetworkstrength.  
 * Written by Gabriel Cain, gabriel-software@dreamingcrow.com
 * This program is distributed and licensed under the GNU GPL.
 */
#include "xns_strength.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <assert.h>

struct gpos_t {
	float lq, sl, nl; /* link quality, signal level, noise level; a number */
			  /* between 0 and 1, multiplied by the height of the  */
			  /* window for the place to draw the line to. */
	};

void DrawWirelessData( struct xns_strength_stat *s );
void DrawTextString();
void BlerbText(void);

#define DoSetProperColor(level)  \
	(level < 0.15 ? xcRed.pixel : \
	( level < 0.40 ? xcYellow.pixel : xcGreen.pixel ))

Display *dpy;
Window w, *wdummy;
int pBlack, pWhite, pGreen, pYellow, pRed;
GC gc;
XEvent e;
XColor xcGreen, xcYellow, xcRed;
char cGreen[] = "#00FF00", cYellow[] = "#FFFF00", cRed[] = "#FF0000";
Colormap colormap;
XFontStruct     *fs;
char fontname[] = "-*-clean-*-r-*-*-8-*-*-*-*-*-*-*";
char *programname = "xnetstrength";
struct gpos_t *gp;
int width, height, x, y;
int tx, ty, tx2, ty2, tx3, ty3, ty4, ty5;
int	tyl, tyl2, tyn, tyn2, tys, tys2;
#define POSCOUNT	52

int main( int argc, char **argv ) {
struct xns_strength_stat *stat;
int i;

BlerbText();

stat = (struct xns_strength_stat*)
	malloc(sizeof(struct xns_strength_stat));
wdummy = (Window *)malloc(sizeof(Window));

gp = (struct gpos_t*)malloc(sizeof(struct gpos_t)*POSCOUNT);
if( !gp ) {
	printf("Cannot allocate position structure.  Aborting.\n");
	exit(45);
	}

for( i = 0; i<POSCOUNT;i++) {
	gp[i].lq = gp[i].sl = gp[i].nl = 0.00;
	}

if( !stat ) {
	printf("Cannot allocate xns_strength structure.  Aborting.\n");
	exit(44);
	}

dpy = XOpenDisplay(NULL);
assert(dpy);

pBlack = BlackPixel( dpy, DefaultScreen(dpy));
pWhite = WhitePixel( dpy, DefaultScreen(dpy));

colormap = DefaultColormap( dpy, 0 );

XParseColor( dpy, colormap, cGreen, &xcGreen );
XAllocColor( dpy, colormap, &xcGreen );

XParseColor( dpy, colormap, cYellow, &xcYellow );
XAllocColor( dpy, colormap, &xcYellow );

XParseColor( dpy, colormap, cRed, &xcRed );
XAllocColor( dpy, colormap, &xcRed );

w = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy), 0, 0, 360, 120, 0, 
	pBlack, pBlack );

XSelectInput( dpy, w, 
	StructureNotifyMask | ExposureMask | KeyPressMask | KeyRelease );

XMapWindow( dpy, w );

/* Add properties, so that we can do fun stuff, like setting the
title, and that crap. */



XmbSetWMProperties( dpy, w, 
	programname,
	programname,
	argv, 
	argc, 
	NULL,		/* We don't use hints. */
	NULL, 
	NULL 
	);

gc = XCreateGC( dpy, w, 0, NULL );
XSetForeground( dpy, gc, pWhite );

fs = XLoadQueryFont( dpy, fontname );
if( !fs ) {
        perror("Font load failed");
	exit(1);
        }

XSetFont( dpy, gc, fs->fid );

for( ;; ) {
	XNextEvent( dpy, &e );
	if( e.type == MapNotify ) 
		break;
	}

while(1) {  /* Our "main loop." */
	GetPNWStats( stat );
	XClearWindow( dpy, w );

	for( i=POSCOUNT-1;i >=0;i--) {
		gp[i+1] = gp[i];
		}
	gp[0].lq = (float)stat->link / 92;
	gp[0].sl = (float)stat->level / 153;
	gp[0].nl = (float)stat->noise / 153;

	XGetGeometry( dpy, w, wdummy, &x, &y, &width, &height, &i, &i ); 


	/* Draw points at the heights... */
	tx = 0; 
	tx2 = 1 *((float)width / POSCOUNT );

	tyl = gp[0].lq * height;
	tyl2 = gp[1].lq * height;
	tyl = height - tyl;
	tyl2 = height - tyl2;
	XSetForeground( dpy, gc, DoSetProperColor( gp[0].lq ));	
	XDrawLine(dpy, w, gc, tx, tyl, tx2, tyl2 );

	tys = gp[0].sl * height;
	tys2 =gp[1].sl * height;
	tys = height - tys;
	tys2 = height - tys2;
	XSetForeground( dpy, gc, DoSetProperColor(gp[0].sl ));
	XDrawLine(dpy, w, gc, tx, tys, tx2, tys2 );
	
        tyn = gp[0].nl * height;
        tyn2 =gp[1].nl * height;
	tyn = height - tyn;
	tyn2 = height- tyn2;
        XSetForeground( dpy, gc, DoSetProperColor(gp[0].sl ));
        XDrawLine(dpy, w, gc, tx, tyn, tx2, tyn2 );

	for(i=1;i<POSCOUNT;i++) {
		tx = tx2;
		tyl = tyl2;
		tyn = tyn2;
		tys = tys2;
		
		tx2 = i*((float)width / POSCOUNT );
		tyl2 = gp[i].lq * height;
		tyn2 = gp[i].nl * height;
		tys2 = gp[i].sl * height; 

		tyl2 = height - tyl2;
		tyn2 = height - tyn2;
		tys2 = height - tys2;


    	        XSetForeground( dpy, gc, DoSetProperColor( gp[i].lq ));
	        XDrawLine(dpy, w, gc, tx, tyl, tx2, tyl2 );
	        XSetForeground( dpy, gc, DoSetProperColor(gp[i].sl ));
	        XDrawLine(dpy, w, gc, tx, tys, tx2, tys2 );
	        XSetForeground( dpy, gc, DoSetProperColor(gp[i].sl ));
	        XDrawLine(dpy, w, gc, tx, tyn, tx2, tyn2 );

		}		


	DrawTextString();
	DrawWirelessData(stat);
	XFlush(dpy);
	usleep( 100000 );
	}


return 0;
}

void DrawWirelessData(struct xns_strength_stat *s ){
char buffer[1024];
XSetForeground( dpy, gc, xcGreen.pixel );
sprintf(buffer,"link quality: %d/92", s->link );
XDrawString( dpy, w, gc, 1, 17, buffer, strlen(buffer) );
XSetForeground( dpy, gc, xcYellow.pixel );
sprintf(buffer,"signal level: %d/153", s->level );
XDrawString( dpy, w, gc, 1, 25, buffer, strlen(buffer) );
XSetForeground( dpy, gc, xcRed.pixel );
sprintf(buffer," noise level: %d/153", s->noise );
XDrawString( dpy, w, gc, 1, 33, buffer, strlen(buffer));
}

void DrawTextString( void ) {
XSetForeground( dpy, gc, pWhite );
XDrawString( dpy, w, gc, 1, 9, "wireless:", strlen("wireless") );
}

void BlerbText(void){
printf(
 "XNetworkStrength - shows stats for devices that write to /proc/net/wireless\n");
printf(
  "Written by Gabriel Cain, gabriel-software@dreamingcrow.com / Dec 8th, 2001\n");
printf(
           "This program is Free Software, and is released under the GNU GPL.\n");
}
